{% extends "manage/manage_base.html" %}
{% load i18n %}

{% block help_link %}user/management/html/static_block.html{% endblock %}
{% block section %}add-static-block{% endblock %}
{% block left-slot-wrapper %}{% endblock %}

{% block content %}
    <div id="manage-tabs">
        <ul>
            <li class="ui-tabs-nav-item"><a href="#data">{% trans 'Add Static Block' %}</a></li>
        </ul>
        <div id="data">
            <h2>
                {% trans 'Data' %}
            </h2>
            <form method="post">
                {% csrf_token %}
                {% include "manage/lfs_form.html" %}
                <input type="submit"
                       class="button"
                       value="{% trans 'Save static block' %}">
                <input type="button"
                       class="button"
                       onclick="window.location='{{ came_from }}'"
                       value="{% trans 'Cancel' %}" />
            </form>
        </div>
    </div>
    <script type="text/javascript">
        tinyMCE.execCommand('mceAddControl', true, 'id_html');
    </script>
{% endblock %}
