{% extends "manage/manage_base.html" %}
{% load i18n %}
{% load lfs_tags %}

{% block help_link %}user/management/customer/reviews.html{% endblock %}
{% block left-slot-wrapper %}{% endblock %}

{% block content %}
    <div id="reviews-filters-inline">
        {{ reviews_filters_inline|safe }}
    </div>
    <div id="manage-tabs">
        <ul>
            <li class="ui-tabs-nav-item"><a href="#reviews">{% trans 'Reviews' %}</a></li>
        </ul>
        <div id="reviews">
            <h2>{% trans "Reviews" %}</h2>
            <div id="reviews-inline">
                {{ reviews_inline|safe }}
            </div>
        </div>
    </div>
{% endblock %}
