{% load i18n %}
<h2 class="navigation-title">
    {% trans "Pages" %}
</h2>
<div class="navigation-body">
    <div class="top-page">
        <a href="{% url lfs_manage_page 1 %}"
           class="selectable {% if page.id == 1 %}selected{% endif %}">
            {{ root.title }}
        </a>
    </div>
    <ul class="sortable"
        href="{% url lfs_sort_pages %}">
        {% for p in pages %}
            <li id="page_{{ p.id }}">
                <div>
                    <a href="{% url lfs_manage_page p.id %}"
                       class="selectable {% if p.id == page.id %}selected{% endif %}">
                       <img class="handle" src="{{ STATIC_URL }}img/handle.png" alt="Handle" />
                        {{ p.title }}
                    </a>
                </div>
            </li>
        {% endfor %}
    </ul>
</div>
<script>sortable()</script>
